import dnf.cli.term
from dnf.i18n import fill_exact_width, textwrap_fill, exact_width, select_short_long, C_, ucd
from dnf.cli.output import Output
from utils import translate


class ABIOutput(object):
    """格式化输出检查结果"""

    def __init__(self, packages_info):
        self.packages_info = packages_info
        self.term = dnf.cli.term.Term()
        self.output = Output(None, None)
        self.output.term = self.term

    def _add_line(self, lines, data, pkg_info):
        """将每一个包信息添加到lines中"""
        pkg_name = pkg_info.name
        compatibility = pkg_info.compatibility_abi
        is_compatible = pkg_info.is_compatible
        undo = pkg_info.undo
        for severity in pkg_info.erratas:
            errara_severity = severity
            for errata in pkg_info.erratas[severity]:
                if len(errata) == 0:
                    continue
                errata_name = list(errata.keys())[0]
                # 如果存在兼容性问题和无法回退，放到最后展示
                if all([is_compatible, undo]):
                    lines.insert(0, (pkg_name, errata_name, errara_severity, compatibility, is_compatible, undo))
                else:
                    lines.append((pkg_name, errata_name, errara_severity, compatibility, is_compatible, undo))
                for (d, v) in (
                        ("name", len(pkg_name)), ("errata", len(errata_name)), ("severity", len(errara_severity)),
                        ("compatibility", len(str(compatibility))), ("is_compatible", len(str(is_compatible))),
                        ("undo", len(str(undo)))):
                    data[d].setdefault(v, 0)
                    data[d][v] += 1

        if not pkg_info.erratas:
            if all([is_compatible, undo]):
                lines.insert(0, (pkg_name, "-", "-", compatibility, is_compatible, undo))
            else:
                lines.append((pkg_name, "-", "-", compatibility, is_compatible, undo))
            for (d, v) in (("name", len(pkg_name)), ("errata", len('-')), ("severity", len('-')),
                           ("compatibility", len(str(compatibility))), ("is_compatible", len(str(is_compatible))),
                           ("undo", len(str(undo)))):
                data[d].setdefault(v, 0)
                data[d][v] += 1

    def get_output(self):
        """得到格式化后的可在终端输出的信息"""
        pkglist_lines = []

        output_width = self.term.columns
        data = {'name': {}, 'errata': {}, 'severity': {}, 'compatibility': {}, 'is_compatible': {}, 'undo': {}}

        lines = []
        for pkg_info in self.packages_info:
            self._add_line(lines, data, pkg_info)
        data = [data['name'], data['errata'], data['severity'], data['compatibility'], data['is_compatible'],
                data['undo']]

        pkglist_lines.append(lines)

        columns = [1, 1, 1, 1, 0, 0]
        columns = self.output.calcColumns(data, indent="  ", columns=columns,
                                          remainder_column=2, total_width=None)
        (name_wid, errata_wid, severity_wid, compatibility_wid, is_compatible_wid, undo_wid) = columns
        real_width = sum(columns) + 5
        output_width = output_width if output_width >= real_width else real_width

        msg_package = select_short_long(name_wid, C_('short', translate('Package')), C_('long', translate('Package')))
        msg_errata = select_short_long(errata_wid, C_('short', translate('Errata')), C_('long', translate('Errata')))
        msg_severity = select_short_long(severity_wid, C_('short', translate('Severity')),
                                         C_('long', translate('Severity')))
        msg_compatibility = select_short_long(compatibility_wid, C_('short', translate('Compatibility')),
                                              C_('long', translate('Compatibility')))
        msg_is_compatible = select_short_long(is_compatible_wid, C_('short', translate('Compatible')),
                                              C_('long', translate('Compatible')))
        msg_undo = select_short_long(undo_wid, C_('short', translate('Undo')), C_('long', translate('Undo')))

        # 格式化输出的头部信息
        out = [u"%s\n%s\n%s\n" % ('=' * output_width, self.output.fmtColumns(((msg_package, -name_wid),
                                                                              (msg_errata, -errata_wid),
                                                                              (msg_severity, -severity_wid),
                                                                              (msg_compatibility, -compatibility_wid),
                                                                              (msg_is_compatible, -is_compatible_wid),
                                                                              (msg_undo, undo_wid)), u" "),
                                  '=' * output_width)]

        # 格式化输出的兼容性信息
        pkg_hi = "bold,green"
        cur_pkg_name = None
        for lines in pkglist_lines:
            totalmsg = ""
            for (name, errata, severity, compatibility, is_compatible, undo) in lines:
                if is_compatible:
                    is_compatible_hi = "bold,green"
                else:
                    is_compatible_hi = "bold,red"

                if undo:
                    undo_hi = "bold,green"
                else:
                    undo_hi = "bold,red"

                if cur_pkg_name is None:
                    cur_pkg_name = name
                    display_name = name
                else:
                    if cur_pkg_name == name:
                        display_name = " " * len(name)
                    else:
                        cur_pkg_name = name
                        display_name = name

                columns = ((display_name, -name_wid, pkg_hi), (errata, -errata_wid),
                           (translate(severity), -severity_wid), (compatibility, -compatibility_wid),
                           (translate(is_compatible), -is_compatible_wid, is_compatible_hi),
                           (translate(undo), undo_wid, undo_hi))
                msg = self.output.fmtColumns(columns, u" ", u"\n")

                totalmsg = totalmsg + msg

            if lines:
                out.append(totalmsg)

        # 格式化输出的尾部信息
        out.append(u"%s\n" % ('=' * output_width))

        return ''.join(out)
